
/*
 * clientSide.c
 *
 * All of these routines are for the client side, ie, user of the API. This
 * will be called to initialise the socket to the server GUI and exchange
 * messages. Some formatting will be done to translate the system call 
 * parameters into messages to the GUI.
 *
 * Message formats are defined in slabmessages.h
 */
#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>

#include <strings.h>
#include <errno.h>

#include "socketToolKit.h"
#include "slabmessages.h"

#define SOCKTYPE SOCK_DGRAM
#define	PROTOCOL "udp"
/*
 * For the service name, defaults to "slab", and can be entered in services
 * file (although this is not implemented yet!).
 */
#define NAME_LENGTH 32

int		socket_descriptor, recvFlag;

void	clearup();
void	clearout();
void	sig_seg();
void	clientCheckSocket();

initControlAPI(host, port, flags)
char	*host;
{
int					namelen;
struct	sockaddr_in	connect_socket_addr;
char				message_buff[256], *read_result_1,  *read_result_2;
char				hostname[NAME_LENGTH], servicename[NAME_LENGTH];
struct	servent		*service, service_tmp;
struct	hostent		*hstp;
u_long				**addrlist;

	/*
	 * Set up our default parameters.
	 */
	(void) sprintf(servicename, "%s", "slab");
	(void) gethostname(hostname, NAME_LENGTH);

	/* 
	 * See if we have been given new ones.
	 */
	if (host != (char *) NULL)
		strcpy(hostname, host);
	if (port == 0)
		port = 5000;

	/*
	 * Check out a few default signals. This should be dropped, since they were
	 * actually for some other programmes.
	(void) signal(SIGINT, clearout);
	 */
	(void) signal(SIGPIPE, clearout);
	(void) signal(SIGTERM, SIG_IGN);
	(void) signal(SIGTTIN, SIG_IGN);
	(void) signal(SIGTTOU, SIG_IGN);
	(void) signal(SIGURG, SIG_IGN);
	(void) signal(SIGHUP, SIG_IGN);
	(void) signal(SIGSEGV, sig_seg);
printf("hostname is %s, %s\n", hostname, SERVICE);
	hstp = gethostbyname(hostname);

	/*
	 * Get a socket in the inet domain
	 */
	if ((socket_descriptor = socket(PF_INET, SOCKTYPE, IPPROTO_UDP))
		== -1) {
		perror ( "socket failed" );
		exit(-1);
	}
	/*
	 * Zero the addr struct, set a few parts of it.
	 */
	(void) bzero((char *) &connect_socket_addr, sizeof (connect_socket_addr));

	/*
	 * Add our service specific components. The use of a pointer is actually
	 * superfluous, but is residual from some more complex service lookup
	 * schemes that could be implemented (ie, from /etc/services).
	 */
	service = &service_tmp;
	service->s_port = port;
	service->s_name = SERVICE;

	connect_socket_addr.sin_family = AF_INET;
	connect_socket_addr.sin_port = htons(service->s_port);

printf("port is %i\n", service->s_port);
	/*
	 * Got to check the address parameter for sockaddr. Grey area!
	 */
	if (hstp == NULL) {
		(void) printf("%s: %s", hostname, "Unknown host?!"); /* no such host */
	}

	if (hstp->h_length != sizeof(u_long))
		return(-1);		/* this is fundamental */

	(void) bcopy(hstp->h_addr, (char *) &connect_socket_addr.sin_addr,
				hstp->h_length);
	/*
	 * Attempt a connection on the known socket.
	 */
	if (connect(socket_descriptor, (struct sockaddr *) &connect_socket_addr,
			sizeof(connect_socket_addr)) == -1) {
		perror("connect failed");
		(void) close(socket_descriptor);
		return(-2);
	}

	initInterpreters();

	/*
	 * Mark this as non-blocking.
	signal(SIGUSR2, clientCheckSocket);
	fcntl(socket_descriptor, F_SETOWN, (long) getpid());
	fcntl(socket_descriptor, F_SETSIG, SIGUSR2);
	fcntl(socket_descriptor, F_SETFL, O_NONBLOCK|O_ASYNC);
	 */

	return(socket_descriptor);
}

/*
 * Forward the buffer to the peer socket.
 * This should be put back in the socketToolKit.c, or similar.
 */
sendMsgBuff(sd, buff, size, pid)
slabmessage *buff;
{
	int sendSize, count = 3;
	struct timeval timeout;
	fd_set readfds;

#ifdef DEBUG
	printf("sendMsgBuff(%i, %i)\n", size, pid);
#endif

	buff->length = htons(size);

	recvFlag = -1;

	while (count-- > 0)
	{
		if ((sendSize = send(sd, buff, size, 0)) < 0)
			perror("send failed");
		if (pid > 0)
			kill(pid, SIGUSR2);
		/*
		 * See if anything came back. We use select() for timing. Frames are
		 * resent 3 times.
		 */
		FD_ZERO(&readfds);
		FD_SET(sd, &readfds);
		timeout.tv_sec = 1;
		timeout.tv_usec = 0;

		if (select(sd + 1, &readfds, NULL, NULL, &timeout) != 0)
		{
			if (recv(sd, buff, size, 0) >= 0)
			{
#ifdef DEBUG
				printf("Have response, returning\n");
#endif
				return(sendSize);
			}
		} else {
#ifdef DEBUG
			printf("Resending frame\n");
#endif
		}
	}

#ifdef DEBUG
	printf("Frame transmission failure\n");
#endif

	return(sendSize);
}

void clientCheckSocket()
{
#ifdef DEBUG
	printf("Got a packet?\n");
#endif
	recvFlag = 0;
}

void
clearout()
{
	(void) printf("%s\n", "Tut, Tut...");
}
void
clearup()
{
	close(socket_descriptor);
	exit(0);
}
void
sig_seg()
{
	(void) printf("%s\n", "Connection closed by local host???");
	clearup();
}

/*
 * Get a buffer;
 */
slabmessage *
getSLabBuffer()
{
	slabmessage *buf;

	buf = (slabmessage *) malloc(SLAB_MAX_MSG_SIZE);

	buf->S = 'S';
	buf->L = 'L';
	buf->a = 'a';
	buf->b = 'b';
	buf->version = SLAB_CONTROL_VERSION;

	return(buf);
}

freeSLabBuffer(msg)
char *msg;
{
	free(msg);
}

